-- YGREC8/ALU9/gate_CLA3bn.vhdl
-- created jeu. mars 28 20:25:29 CET 2019 by Yann Guidon (whygee@f-cpu.org)
-- version mer. janv.  1 19:04:18 CET 2020 : version b with inverted E input.
-- version jeu. janv.  2 00:02:30 CET 2020 : version with inverted output (NAND=>NOR)
--
-- Released under the GNU AGPLv3 license
--
-- This "complex gate" is an important building block for a Carry-Lookahead
-- adder circuit. This new version saves one inverter for the ASIC version.

Library ieee;
    use ieee.std_logic_1164.all;
Library SLV;
    use SLV.SLV_utils.all;
Library proasic3;
    use proasic3.all;

entity CLA3BN is
  port( A, B, C, D, E : in SL;
    Y : out SL);
end CLA3BN;

architecture RTL of CLA3BN is
  Signal T2, T3: SL;
begin
  Y <= not(( (not A) AND (not B)  AND (not C))
        OR ( (not A) AND (not D))
        OR E);
end RTL;

architecture A3P of CLA3BN is
  Signal T2, T3: SL;
begin
  cla3_a: entity AND2B port map(A=>A, B=>D,  Y=>T2);        -- T2<= (not A) AND (not D);
  cla3_b: entity AND3C port map(A=>A, B=>B,  C=>C,  Y=>T3); -- T3<= (not A  AND (not B) AND (not C);
  cla3_c: entity NOR3  port map(A=>E, B=>T2, C=>T3, Y=>Y);  -- Y<= not(E or T2 OR T3);
end A3P;

architecture ASIC of CLA3BN is
  Signal T2, T3: SL;
begin
  cla3_a: entity NOR2 port map(A=>A, B=>D,         Y=>T2); -- T2<=not (A or D);
  cla3_b: entity NOR3 port map(A=>A, B=>B,  C=>C,  Y=>T3); -- T3<=not (A or B or C);
  cla3_c: entity NOR3 port map(A=>E, B=>T2, C=>T3, Y=>Y);  --  Y<=not (T2 or T3 or E);
end ASIC;
