#!/bin/bash
# file test.sh
# created by Yann Guidon / ygdes.com
# version sam. nov. 11 19:26:36 CET 2017
# version lun. nov. 13 02:34:03 CET 2017
# version jeu. nov. 16 03:07:00 CET 2017
# version mer. oct. 17 03:38:23 CEST 2018 : INC8_tiles.vhdl & INC8_gates.vhdl
# version mar. nov. 19 06:02:18 CET 2019 : integrated in the proasic3v2 testbench
# version jeu. nov. 21 06:57:25 CET 2019 : trace changes
# version lun. déc. 30 15:48:49 CET 2019 : back from A3Plib to Y8
# version jeu. jul. 16 02:36:53 CEST 2020 : added ASIC version


if [ ! -f ../proasic3v3/proasic3-obj93.cf  ]; then
  echo -n '--> going to ' ; pushd ../proasic3v3/
  ./build.sh
  echo -n '<-- back to ' ; popd
fi

PA3="-P../proasic3v3"

function run_test() {
  # arg.1 : INC8 version/file
  # arg.2 : message
  # optional $PA3
  rm -f inc8 inc8_tb work-obj93.cf *.o
  ghdl -a $PA3           $1 && ghdl -e $PA3 INC8    && ./inc8    && echo INC8 $2 standalone: OK &&
  ghdl -a $PA3 INC8_tb.vhdl && ghdl -e $PA3 INC8_tb && ./inc8_tb && echo INC8 $2 testbench:  OK
}

echo
echo "Sanity check of the reference designs :"
run_test INC8.vhdl        ""      &&
run_test INC8_A3P.vhdl    "A3P"   &&
run_test INC8_gates.vhdl  "gates" &&
run_test INC8_tiles.vhdl  "tiles" &&
run_test INC8_ASIC.vhdl   "ASIC"  &&
echo "   hmmmm noyyyce..." &&
echo &&
echo "OK now let's list the gates" &&
rm -f inc8 inc8_tb work-obj93.cf *.o &&
ghdl -a $PA3 INC8_ASIC.vhdl INC8_tb.vhdl && ghdl -e $PA3 INC8_tb && ./inc8_tb -gflip=0 &&
# note : -gflip=0 enables additional features

echo &&
echo "Then we check that all the errors can be found :"
DEPTH_LIST=$(./inc8_tb -gflip=0 -ggate_select_number=-1 |grep '  gate#' |sed 's/.*(trace):lut//' ) &&
k=1 && # first gate is n°1
l=0 &&
rm -f failures.log &&
# scan the list to extract the depth
for i in $DEPTH_LIST ; do
  # for each bit in the LUT:
  if [ $i \> 2 ]; then  # don't test gates with fewer than 2 inputs
    for j in $(seq 0 $((i-1)) ) ; do
      echo -n $'\r'"$l/83  " >&2
      echo 'Run #'$l  'Gate #'$k 'LUT'$i'['$j']'
      (./inc8_tb -gflip=1 -ggate_select_number=$k -gbit_select_number=$j &&
            echo 'failure not found :-(' ) || echo 'Failure detected : great !'
      echo ""
      l=$((l+1))
    done
  fi
  k=$((k+1))
done > failures.log &&

( grep 'failure not found :-(' failures.log &&
    echo "Oh, some failures have not been detected :-(") ||
      ( echo "" &&
        s=0 &&
        for i in $(grep 'ns:(assertion failure): Error at ' failures.log | sed 's/.*  cycle# //' |sed 's/ .*//' )
        do
          s=$(( s + i ))
        done &&
        echo "All the bits have been successfully checked in $s cycles !" &&
        rm -f inc8 work-obj93.cf failures.log inc8_tb *.o ) &&

echo
echo "Finally let's extract and check the netlist :" &&

ghdl -a $PA3 INC8_ASIC.vhdl  Wrap_INC8.vhdl ../proasic3v3/VectGen.vhdl &&
ghdl -e $PA3 VectGen &&
./vectgen | sed 's/.*te)://' |\
  grep -A 9999 ' END OF DEPTHLIST ' | \
  grep -v '      0 -' |\
  grep -v ' END OF DEPTHLIST '
